@@ -14,7 +14,7 @@
 
 #include <geos/triangulate/DelaunayTriangulationBuilder.h>
 #include <geos/triangulate/VoronoiDiagramBuilder.h>
-#include <geos/geom/CoordinateArraySequence.h>
+#include <geos/geom/CoordinateSequence.h>
 #include <geos/geom/GeometryFactory.h>
 #include <geos/profiler.h>
 
@@ -26,7 +26,7 @@
 class SegmentUnaryUnionPerfTest {
 
 public:
-    void test(size_t num_lines) {
+    void test(std::size_t num_lines) {
         using namespace geos::geom;
 
         std::default_random_engine e(12345);
@@ -35,8 +35,8 @@ public:
 
         std::vector<std::unique_ptr<LineString>> lines;
 
-        for (size_t i = 0; i < num_lines; i++) {
-            CoordinateArraySequence cas(2, 2);
+        for (std::size_t i = 0; i < num_lines; i++) {
+            CoordinateSequence cas(2, 2);
             cas.setAt(Coordinate(dis(e), dis(e)), 0);
             cas.setAt(Coordinate(dis(e), dis(e)), 1);
 
@@ -65,7 +65,7 @@ int main(int argc, char** argv) {
     auto num_lines = std::atol(argv[1]);
     auto num_reps = argc > 2 ? std::atol(argv[2]) : 1;
 
-    for (std::size_t i = 0; i < num_reps; i++) {
-        tester.test(num_lines);
+    for (decltype(num_reps) i = 0; i < num_reps; i++) {
+        tester.test(static_cast<std::size_t>(num_lines));
     }
 }
