@@ -17,14 +17,16 @@
  *
  **********************************************************************/
 
-#ifndef GEOS_IO_WKBWRITER_H
-#define GEOS_IO_WKBWRITER_H
+#pragma once
 
 #include <geos/export.h>
 
 #include <geos/util/Machine.h> // for getMachineByteOrder
+#include <geos/io/OrdinateSet.h>
+#include <geos/io/WKBConstants.h>
 #include <iosfwd>
 #include <cstdint>
+#include <cstddef>
 
 // Forward declarations
 namespace geos {
@@ -56,7 +58,7 @@ namespace io {
  *
  * The WKB format is specified in the OGC Simple Features for SQL specification.
  * This implementation supports the extended WKB standard for representing
- * 3-dimensional coordinates.  The presence of 3D coordinates is signified
+ * Z and M coordinates.  The presence of Z and/or M coordinates is signified
  * by setting the high bit of the wkbType word.
  *
  * Empty Points cannot be represented in WKB; an
@@ -78,27 +80,32 @@ public:
      * Initializes writer with target coordinate dimension, endianness
      * flag and SRID value.
      *
-     * @param dims Supported values are 2 or 3.  Note that 3 indicates
-     * up to 3 dimensions will be written but 2D WKB is still produced for 2D geometries.
+     * @param dims Supported values are 2, 3 or 4.  Note that 4 indicates
+     * up to 4 dimensions will be written but (e.g.) 2D WKB is still produced
+     * for 2D geometries. Default since GEOS 3.12 is 4.
      * @param bo output byte order - default to native machine byte order.
      * Legal values include 0 (big endian/xdr) and 1 (little endian/ndr).
      * @param incudeSRID true if SRID should be included in WKB (an
      * extension).
      */
-    WKBWriter(uint8_t dims = 2, int bo = getMachineByteOrder(), bool includeSRID = false);
+    WKBWriter(
+        uint8_t dims = 4,
+        int bo = getMachineByteOrder(),
+        bool includeSRID = false,
+        int flv = WKBConstants::wkbExtended);
 
     /*
      * \brief
      * Destructor.
      */
-    virtual ~WKBWriter() = default;
+    ~WKBWriter() = default;
 
     /*
      * \brief
      * Returns the output dimension used by the
      * <code>WKBWriter</code>.
      */
-    virtual uint8_t
+    uint8_t
     getOutputDimension() const
     {
         return defaultOutputDimension;
@@ -107,18 +114,18 @@ public:
     /*
      * Sets the output dimension used by the <code>WKBWriter</code>.
      *
-     * @param newOutputDimension Supported values are 2 or 3.
-     * Note that 3 indicates up to 3 dimensions will be written but
-     * 2D WKB is still produced for 2D geometries.
+     * @param newOutputDimension Supported values are 2, 3 or 4.
+     * Note that 4 indicates up to 4 dimensions will be written but
+     * (e.g.) 2D WKB is still produced for 2D geometries.
      */
-    virtual void setOutputDimension(uint8_t newOutputDimension);
+    void setOutputDimension(uint8_t newOutputDimension);
 
     /*
      * \brief
      * Returns the byte order used by the
      * <code>WKBWriter</code>.
      */
-    virtual int
+    int
     getByteOrder() const
     {
         return byteOrder;
@@ -128,14 +135,14 @@ public:
      * Sets the byte order used by the
      * <code>WKBWriter</code>.
      */
-    virtual void setByteOrder(int newByteOrder);
+    void setByteOrder(int newByteOrder);
 
     /*
      * \brief
      * Returns whether SRID values are output by the
      * <code>WKBWriter</code>.
      */
-    virtual bool
+    bool
     getIncludeSRID() const
     {
         return includeSRID;
@@ -145,12 +152,28 @@ public:
      * Sets whether SRID values should be output by the
      * <code>WKBWriter</code>.
      */
-    virtual void
+    void
     setIncludeSRID(bool newIncludeSRID)
     {
         includeSRID = newIncludeSRID;
     }
 
+    /*
+     * \brief
+     * Returns the WKB flavor the writer will emit.
+     */
+    int
+    getFlavor() const
+    {
+        return flavor;
+    }
+
+    /*
+     * \brief
+     * Set the WKB flavor the writer will emit.
+     */
+    void setFlavor(int newFlavor);
+
     /**
      * \brief Write a Geometry to an ostream.
      *
@@ -173,10 +196,14 @@ public:
 
 private:
 
+    // 2, 3, or 4
     uint8_t defaultOutputDimension;
-    uint8_t outputDimension;
+    OrdinateSet outputOrdinates;
 
+    // WKBConstants::wkbwkbXDR | WKBConstants::wkbNDR
     int byteOrder;
+    // WKBConstants::wkbIso | WKBConstants::wkbExtended
+    int flavor;
 
     bool includeSRID;
 
@@ -200,7 +227,7 @@ private:
     void writeCoordinateSequence(const geom::CoordinateSequence& cs, bool sized);
     // throws IOException
 
-    void writeCoordinate(const geom::CoordinateSequence& cs, size_t idx, bool is3d);
+    void writeCoordinate(const geom::CoordinateSequence& cs, std::size_t idx);
     // throws IOException
 
     void writeGeometryType(int geometryType, int SRID);
@@ -215,9 +242,10 @@ private:
     void writeInt(int intValue);
     // throws IOException
 
+    OrdinateSet getOutputOrdinates(OrdinateSet ordinates);
+
 };
 
 } // namespace io
 } // namespace geos
 
-#endif // #ifndef GEOS_IO_WKBWRITER_H
