@@ -38,40 +38,35 @@ namespace locate { // geos.algorithm
  * is more complex, since it has to take into account the boundaryDetermination rule
  */
 geom::Location
-SimplePointInAreaLocator::locate(const Coordinate& p, const Geometry* geom)
+SimplePointInAreaLocator::locate(const CoordinateXY& p, const Geometry* geom)
 {
-    if(geom->isEmpty()) {
-        return Location::EXTERIOR;
-    }
+    return locateInGeometry(p, geom);
+}
 
+bool
+SimplePointInAreaLocator::isContained(const CoordinateXY& p, const Geometry* geom)
+{
+    return Location::EXTERIOR != locate(p, geom);
+}
+
+geom::Location
+SimplePointInAreaLocator::locateInGeometry(const CoordinateXY& p, const Geometry* geom)
+{
     /*
      * Do a fast check against the geometry envelope first
      */
     if (! geom->getEnvelopeInternal()->intersects(p))
         return Location::EXTERIOR;
 
-    return locateInGeometry(p, geom);
-}
-
-bool
-SimplePointInAreaLocator::isContained(const Coordinate& p, const Geometry* geom)
-{
-    return Location::EXTERIOR != locate(p, geom);
-}
-
-geom::Location
-SimplePointInAreaLocator::locateInGeometry(const Coordinate& p, const Geometry* geom)
-{
     if (geom->getDimension() < 2) {
         return Location::EXTERIOR;
     }
 
     if (geom->getNumGeometries() == 1) {
-        auto poly = dynamic_cast<const Polygon*>(geom->getGeometryN(0));
-        if (poly) {
+        if (geom->getGeometryTypeId() == GEOS_POLYGON) {
+            auto poly = static_cast<const Polygon*>(geom);
             return locatePointInPolygon(p, poly);
         }
-        // Else it is a collection with a single element. Will be handled below.
     }
     for (std::size_t i = 0; i < geom->getNumGeometries(); i++) {
         const Geometry* gi = geom->getGeometryN(i);
@@ -85,7 +80,7 @@ SimplePointInAreaLocator::locateInGeometry(const Coordinate& p, const Geometry*
 }
 
 geom::Location
-SimplePointInAreaLocator::locatePointInPolygon(const Coordinate& p, const Polygon* poly)
+SimplePointInAreaLocator::locatePointInPolygon(const CoordinateXY& p, const Polygon* poly)
 {
     if(poly->isEmpty()) {
         return Location::EXTERIOR;
