@@ -10,10 +10,6 @@
  * by the Free Software Foundation.
  * See the COPYING file for more information.
  *
- **********************************************************************
- *
- * Last port: noding/MCIndexSegmentSetMutualIntersector.java r388 (JTS-1.12)
- *
  **********************************************************************/
 
 #include <geos/noding/MCIndexSegmentSetMutualIntersector.h>
@@ -25,6 +21,7 @@
 #include <geos/index/chain/MonotoneChainBuilder.h>
 #include <geos/index/chain/MonotoneChainOverlapAction.h>
 #include <geos/index/strtree/SimpleSTRtree.h>
+
 // std
 #include <cstddef>
 
@@ -33,21 +30,25 @@ using namespace geos::index::chain;
 namespace geos {
 namespace noding { // geos::noding
 
+
 /*private*/
 void
 MCIndexSegmentSetMutualIntersector::addToIndex(SegmentString* segStr)
 {
-    MonoChains segChains;
     MonotoneChainBuilder::getChains(segStr->getCoordinates(),
-                                    segStr, segChains);
+                                    segStr, indexChains);
 
-    MonoChains::size_type n = segChains.size();
-    chainStore.reserve(chainStore.size() + n);
-    for(auto& mc : segChains) {
-        mc->setId(indexCounter++);
-        index->insert(&(mc->getEnvelope()), mc.get());
-        chainStore.push_back(std::move(mc));
-    }
+}
+
+
+/*private*/
+void
+MCIndexSegmentSetMutualIntersector::addToMonoChains(SegmentString* segStr)
+{
+    if (segStr->size() == 0)
+        return;
+    MonotoneChainBuilder::getChains(segStr->getCoordinates(),
+                                    segStr, monoChains);
 }
 
 
@@ -57,55 +58,16 @@ MCIndexSegmentSetMutualIntersector::intersectChains()
 {
     MCIndexSegmentSetMutualIntersector::SegmentOverlapAction overlapAction(*segInt);
 
-    std::vector<void*> overlapChains;
-    for(const auto& queryChain : monoChains) {
-        overlapChains.clear();
-
-        index->query(&(queryChain->getEnvelope()), overlapChains);
-
-        for(std::size_t j = 0, nj = overlapChains.size(); j < nj; j++) {
-            MonotoneChain* testChain = (MonotoneChain*)(overlapChains[j]);
-
-            queryChain->computeOverlaps(testChain, &overlapAction);
+    for(auto& queryChain : monoChains) {
+        index.query(queryChain.getEnvelope(overlapTolerance), [&queryChain, &overlapAction, this](const MonotoneChain* testChain) -> bool {
+            queryChain.computeOverlaps(testChain, overlapTolerance, &overlapAction);
             nOverlaps++;
-            if(segInt->isDone()) {
-                return;
-            }
-        }
+
+            return !segInt->isDone(); // abort early if segInt->isDone()
+        });
     }
 }
 
-/*private*/
-void
-MCIndexSegmentSetMutualIntersector::addToMonoChains(SegmentString* segStr)
-{
-    MonoChains segChains;
-    MonotoneChainBuilder::getChains(segStr->getCoordinates(),
-                                    segStr, segChains);
-
-    MonoChains::size_type n = segChains.size();
-    monoChains.reserve(monoChains.size() + n);
-    for(auto& mc : segChains) {
-        mc->setId(processCounter++);
-        monoChains.push_back(std::move(mc));
-    }
-}
-
-/* public */
-MCIndexSegmentSetMutualIntersector::MCIndexSegmentSetMutualIntersector()
-    :	monoChains(),
-      index(new geos::index::strtree::SimpleSTRtree()),
-      indexCounter(0),
-      processCounter(0),
-      nOverlaps(0)
-{
-}
-
-/* public */
-MCIndexSegmentSetMutualIntersector::~MCIndexSegmentSetMutualIntersector()
-{
-    delete index;
-}
 
 /* public */
 void
@@ -113,8 +75,9 @@ MCIndexSegmentSetMutualIntersector::setBaseSegments(SegmentString::ConstVect* se
 {
     // NOTE - mloskot: const qualifier is removed silently, dirty.
 
-    for(std::size_t i = 0, n = segStrings->size(); i < n; i++) {
-        const SegmentString* css = (*segStrings)[i];
+    for(const SegmentString* css: *segStrings) {
+        if (css->size() == 0)
+            continue;
         SegmentString* ss = const_cast<SegmentString*>(css);
         addToIndex(ss);
     }
@@ -124,14 +87,21 @@ MCIndexSegmentSetMutualIntersector::setBaseSegments(SegmentString::ConstVect* se
 void
 MCIndexSegmentSetMutualIntersector::process(SegmentString::ConstVect* segStrings)
 {
+    if (!indexBuilt) {
+        for (auto& mc: indexChains) {
+            index.insert(&(mc.getEnvelope(overlapTolerance)), &mc);
+        }
+        indexBuilt = true;
+    }
+
+    // Reset counters for new inputs
+    monoChains.clear();
     processCounter = indexCounter + 1;
     nOverlaps = 0;
 
-    monoChains.clear();
-
-    for(SegmentString::ConstVect::size_type i = 0, n = segStrings->size(); i < n; i++) {
-        SegmentString* seg = (SegmentString*)((*segStrings)[i]);
-        addToMonoChains(seg);
+    for(const SegmentString* css: *segStrings) {
+        SegmentString* ss = const_cast<SegmentString*>(css);
+        addToMonoChains(ss);
     }
     intersectChains();
 }
@@ -140,14 +110,13 @@ MCIndexSegmentSetMutualIntersector::process(SegmentString::ConstVect* segStrings
 /* public */
 void
 MCIndexSegmentSetMutualIntersector::SegmentOverlapAction::overlap(
-    MonotoneChain& mc1, size_t start1, MonotoneChain& mc2, size_t start2)
+    const MonotoneChain& mc1, std::size_t start1, const MonotoneChain& mc2, std::size_t start2)
 {
-    SegmentString* ss1 = (SegmentString*)(mc1.getContext());
-    SegmentString* ss2 = (SegmentString*)(mc2.getContext());
+    SegmentString* ss1 = static_cast<SegmentString*>(mc1.getContext());
+    SegmentString* ss2 = static_cast<SegmentString*>(mc2.getContext());
 
     si.processIntersections(ss1, start1, ss2, start2);
 }
 
 } // geos::noding
 } // geos
-
