@@ -15,11 +15,14 @@
 #include <geos/operation/overlayng/CoverageUnion.h>
 
 #include <geos/noding/SegmentExtractingNoder.h>
+#include <geos/noding/BoundaryChainNoder.h>
 #include <geos/operation/overlayng/OverlayNG.h>
 #include <geos/geom/Geometry.h>
+#include <geos/util/TopologyException.h>
 
 using geos::geom::Geometry;
 using geos::noding::SegmentExtractingNoder;
+using geos::noding::BoundaryChainNoder;
 
 namespace geos {      // geos
 namespace operation { // geos.operation
@@ -30,10 +33,27 @@ namespace overlayng { // geos.operation.overlayng
 std::unique_ptr<Geometry>
 CoverageUnion::geomunion(const Geometry* coverage)
 {
-    SegmentExtractingNoder sen;
+    double area_in = coverage->getArea();
+    std::unique_ptr<Geometry> result;
 
     // a precision model is not needed since no noding is done
-    return OverlayNG::geomunion(coverage, nullptr, &sen);
+    //-- linear networks require a segment-extracting noder
+    if (coverage->getDimension() < 2) {
+        SegmentExtractingNoder sen;
+        result = OverlayNG::geomunion(coverage, nullptr, &sen);
+    }
+    else {
+        BoundaryChainNoder bcn;
+        result = OverlayNG::geomunion(coverage, nullptr, &bcn);
+    }
+
+    double area_out = result->getArea();
+
+    if (std::abs((area_out - area_in)/area_in) > AREA_PCT_DIFF_TOL) {
+        throw geos::util::TopologyException("CoverageUnion cannot process overlapping inputs.");
+    }
+
+    return result;
 }
 
 
