@@ -20,15 +20,16 @@
 #include <geos/precision/GeometryPrecisionReducer.h>
 #include <geos/precision/PrecisionReducerCoordinateOperation.h>
 #include <geos/geom/util/GeometryEditor.h>
-#include <geos/geom/util/CoordinateOperation.h>
+#include <geos/geom/util/NoOpGeometryOperation.h>
 #include <geos/geom/Coordinate.h>
 #include <geos/geom/CoordinateSequence.h>
-#include <geos/geom/CoordinateSequenceFactory.h>
 #include <geos/geom/PrecisionModel.h>
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/LineString.h>
 #include <geos/geom/LinearRing.h>
 #include <geos/operation/overlayng/PrecisionReducer.h>
+#include <geos/precision/PrecisionReducerTransformer.h>
+#include <geos/precision/PointwisePrecisionReducerTransformer.h>
 
 #include <vector>
 #include <typeinfo>
@@ -44,66 +45,73 @@ namespace precision { // geos.precision
 std::unique_ptr<Geometry>
 GeometryPrecisionReducer::reduce(const Geometry& geom)
 {
-    if (useAreaReducer && geom.isPolygonal()) {
-        std::unique_ptr<Geometry> reduced =
-            operation::overlayng::PrecisionReducer::reducePrecision(
-                &geom, &targetPM, changePrecisionModel);
-        // GEOS overlayng::PrecisionReducer::reducePrecision
-        // takes changePrecisionModel directly to avoid extra steps in JTS
-        // if (changePrecisionModel) {
-        //     return changePM(reduced, targetPM);
-        // }
-        return reduced;
+    std::unique_ptr<Geometry> reduced;
+    if (isPointwise) {
+        reduced = PointwisePrecisionReducerTransformer::reduce(geom, targetPM);
+    }
+    else {
+        reduced = PrecisionReducerTransformer::reduce(geom, targetPM, removeCollapsed);
     }
 
-    /**
-     * Process pointwise reduction
-     * (which is only strategy used for linear and point geoms)
-     */
-    std::unique_ptr<Geometry> reducePW = reducePointwise(geom);
-
-    if (isPointwise)
-        return reducePW;
-
-    if(!reducePW->isPolygonal()) {
-        return reducePW;
+    // Match the collection level of the output to the input
+    // if necessary
+    if (geom.isCollection()
+        && ! reduced->isCollection()
+        && (geom.getCoordinateDimension() == reduced->getCoordinateDimension()))
+    {
+        reduced = geom.getFactory()->createMulti(std::move(reduced));
     }
 
-    // Geometry is polygonal - test if topology needs to be fixed
-    if(reducePW->isValid()) {
-        return reducePW;
+    // TODO: incorporate this in the Transformer above
+    if (changePrecisionModel &&
+        (&targetPM != geom.getFactory()->getPrecisionModel()))
+    {
+         return changePM(reduced.get(), targetPM);
     }
 
-    // hack to fix topology.
-    return fixPolygonalTopology(*reducePW);
+    return reduced;
 }
 
 
 /* private */
 std::unique_ptr<Geometry>
-GeometryPrecisionReducer::reducePointwise(const Geometry& geom)
+GeometryPrecisionReducer::changePM(const Geometry* geom, const geom::PrecisionModel& newPM)
 {
-    std::unique_ptr<GeometryEditor> geomEdit;
+    const GeometryFactory* previousFactory = geom->getFactory();
+    GeometryFactory::Ptr changedFactory = createFactory(*previousFactory, newPM);
+    GeometryEditor geomEdit(changedFactory.get());
 
-    if (changePrecisionModel) {
-        geomEdit.reset(new GeometryEditor(newFactory));
-    }
-    else {
-        geomEdit.reset(new GeometryEditor());
-    }
+    // this operation changes the PM for the entire geometry tree
+    NoOpGeometryOperation noop;
+    return geomEdit.edit(geom, &noop);
+}
 
-    /*
-     * For polygonal geometries, collapses are always removed, in order
-     * to produce correct topology
-     */
-    bool finalRemoveCollapsed = removeCollapsed;
-    if(geom.getDimension() >= 2) {
-        finalRemoveCollapsed = true;
-    }
 
-    PrecisionReducerCoordinateOperation prco(targetPM, finalRemoveCollapsed);
-    std::unique_ptr<Geometry> g(geomEdit->edit(&geom, &prco));
-    return g;
+/* public static */
+std::unique_ptr<Geometry>
+GeometryPrecisionReducer::reducePointwise(const Geometry& g, const geom::PrecisionModel& precModel)
+{
+    GeometryPrecisionReducer reducer(precModel);
+    reducer.setPointwise(true);
+    return reducer.reduce(g);
+}
+
+/* public static */
+std::unique_ptr<Geometry>
+GeometryPrecisionReducer::reduceKeepCollapsed(const Geometry& g, const geom::PrecisionModel& precModel)
+{
+    GeometryPrecisionReducer reducer(precModel);
+    reducer.setRemoveCollapsedComponents(false);
+    return reducer.reduce(g);
+}
+
+
+/* public static */
+std::unique_ptr<geom::Geometry>
+GeometryPrecisionReducer::reduce(const geom::Geometry& g, const geom::PrecisionModel& precModel)
+{
+    GeometryPrecisionReducer reducer(precModel);
+    return reducer.reduce(g);
 }
 
 
@@ -122,7 +130,7 @@ GeometryPrecisionReducer::fixPolygonalTopology(const Geometry& geom)
 
     if(! newFactory) {
         tmpFactory = createFactory(*geom.getFactory(), targetPM);
-        tmp.reset(tmpFactory->createGeometry(&geom));
+        tmp = tmpFactory->createGeometry(&geom);
         geomToBuffer = tmp.get();
     }
 
@@ -130,7 +138,7 @@ GeometryPrecisionReducer::fixPolygonalTopology(const Geometry& geom)
 
     if(! newFactory) {
         // a slick way to copy the geometry with the original precision factory
-        bufGeom.reset(geom.getFactory()->createGeometry(bufGeom.get()));
+        bufGeom = geom.getFactory()->createGeometry(bufGeom.get());
     }
 
     return bufGeom;
@@ -143,8 +151,7 @@ GeometryPrecisionReducer::createFactory(const GeometryFactory& oldGF,
 {
     GeometryFactory::Ptr p_newFactory(
         GeometryFactory::create(&newPM,
-                                oldGF.getSRID(),
-                                const_cast<CoordinateSequenceFactory*>(oldGF.getCoordinateSequenceFactory()))
+                                oldGF.getSRID())
     );
     return p_newFactory;
 }
@@ -153,4 +160,3 @@ GeometryPrecisionReducer::createFactory(const GeometryFactory& oldGF,
 
 } // namespace geos.precision
 } // namespace geos
-
