@@ -23,7 +23,7 @@
 #include <geos/geom/LinearRing.h>
 #include <geos/geom/Geometry.h> // for unique_ptr destructor
 #include <geos/geom/GeometryFactory.h>
-#include <geos/geom/CoordinateSequenceFactory.h>
+#include <geos/util.h>
 
 #include <cassert>
 #include <memory>
@@ -32,22 +32,24 @@
 #define GEOS_DEBUG 0
 #endif
 
-#ifdef GEOS_DEBUG
+#if GEOS_DEBUG
 #include <iostream>
 #endif
 
 using namespace geos::geom;
-using namespace std;
+
 
 namespace geos {
 namespace simplify { // geos::simplify
 
 /*public*/
 TaggedLineString::TaggedLineString(const geom::LineString* nParentLine,
-                                   std::size_t nMinimumSize)
+                                   std::size_t nMinimumSize,
+                                   bool bPreserveEndpoint)
     :
     parentLine(nParentLine),
-    minimumSize(nMinimumSize)
+    minimumSize(nMinimumSize),
+    preserveEndpoint(bPreserveEndpoint)
 {
     init();
 }
@@ -56,8 +58,8 @@ TaggedLineString::TaggedLineString(const geom::LineString* nParentLine,
 TaggedLineString::~TaggedLineString()
 {
 #if GEOS_DEBUG
-    cerr << "TaggedLineString[" << this << "] destructor"
-         << endl;
+    std::cerr << "TaggedLineString[" << this << "] destructor"
+         << std::endl;
 #endif
 
     for(std::size_t i = 0, n = segs.size(); i < n; i++) {
@@ -77,8 +79,8 @@ TaggedLineString::init()
     const CoordinateSequence* pts = parentLine->getCoordinatesRO();
 
 #if GEOS_DEBUG
-    cerr << "TaggedLineString[" << this << "] pts.size() " << pts->size()
-         << endl;
+    std::cerr << "TaggedLineString[" << this << "] pts.size() " << pts->size()
+         << std::endl;
 #endif
 
     if(!pts->isEmpty()) {
@@ -97,10 +99,10 @@ TaggedLineString::init()
     }
 
 #if GEOS_DEBUG
-    cerr << "TaggedLineString[" << this << "] segs.size " << segs.size()
-         << endl;
-    cerr << "TaggedLineString[" << this << "] resultSegs.size " << resultSegs.size()
-         << endl;
+    std::cerr << "TaggedLineString[" << this << "] segs.size " << segs.size()
+         << std::endl;
+    std::cerr << "TaggedLineString[" << this << "] resultSegs.size " << resultSegs.size()
+         << std::endl;
 #endif
 }
 
@@ -111,6 +113,13 @@ TaggedLineString::getMinimumSize() const
     return minimumSize;
 }
 
+/*public*/
+bool
+TaggedLineString::getPreserveEndpoint() const
+{
+    return preserveEndpoint;
+}
+
 /*public*/
 const geom::LineString*
 TaggedLineString::getParent() const
@@ -132,45 +141,42 @@ TaggedLineString::getResultCoordinates() const
 {
 
 #if GEOS_DEBUG
-    cerr << __FUNCTION__ << " resultSegs.size: "
-         << resultSegs.size() << endl;
+    std::cerr << __FUNCTION__ << " resultSegs.size: "
+         << resultSegs.size() << std::endl;
 #endif
 
-    CoordVectPtr pts = extractCoordinates(resultSegs);
+    auto pts = extractCoordinates(resultSegs);
 
 #if GEOS_DEBUG
-    cerr << __FUNCTION__ << " extracted Coords.size: "
-         << pts->size() << endl;
+    std::cerr << __FUNCTION__ << " extracted Coords.size: "
+         << pts->size() << std::endl;
 #endif
 
-
-    CoordVect* v = pts.release();
-    return CoordinateSequence::Ptr(parentLine->getFactory()->getCoordinateSequenceFactory()->create(v));
-
+    return pts;
 }
 
 /*private static*/
-TaggedLineString::CoordVectPtr
+std::unique_ptr<CoordinateSequence>
 TaggedLineString::extractCoordinates(
     const std::vector<TaggedLineSegment*>& segs)
 {
-    CoordVectPtr pts(new CoordVect());
+    auto pts = detail::make_unique<geom::CoordinateSequence>();
 
 #if GEOS_DEBUG
-    cerr << __FUNCTION__ << " segs.size: " << segs.size() << endl;
+    std::cerr << __FUNCTION__ << " segs.size: " << segs.size() << std::endl;
 #endif
 
-    std::size_t i = 0, size = segs.size();
+    const std::size_t size = segs.size();
 
     if(size) {
-        for(; i < size; i++) {
+        for(std::size_t i = 0; i < size; i++) {
             TaggedLineSegment* seg = segs[i];
             assert(seg);
-            pts->push_back(seg->p0);
+            pts->add(seg->p0);
         }
 
         // add last point
-        pts->push_back(segs[size - 1]->p1);
+        pts->add(segs[size - 1]->p1);
     }
 
     return pts;
@@ -199,22 +205,28 @@ TaggedLineString::getSegment(std::size_t i) const
 }
 
 /*public*/
-vector<TaggedLineSegment*>&
+std::vector<TaggedLineSegment*>&
 TaggedLineString::getSegments()
 {
-    assert(0);
     return segs;
 }
 
 /*public*/
-const vector<TaggedLineSegment*>&
+const std::vector<TaggedLineSegment*>&
 TaggedLineString::getSegments() const
 {
     return segs;
 }
 
 /*public*/
-unique_ptr<Geometry>
+const std::vector<TaggedLineSegment*>&
+TaggedLineString::getResultSegments() const
+{
+    return resultSegs;
+}
+
+/*public*/
+std::unique_ptr<Geometry>
 TaggedLineString::asLineString() const
 {
     return parentLine->getFactory()->createLineString(
@@ -222,7 +234,7 @@ TaggedLineString::asLineString() const
 }
 
 /*public*/
-unique_ptr<Geometry>
+std::unique_ptr<Geometry>
 TaggedLineString::asLinearRing() const
 {
     return std::unique_ptr<Geometry>(parentLine->getFactory()->createLinearRing(
@@ -231,20 +243,31 @@ TaggedLineString::asLinearRing() const
 
 /*public*/
 void
-TaggedLineString::addToResult(unique_ptr<TaggedLineSegment> seg)
+TaggedLineString::addToResult(std::unique_ptr<TaggedLineSegment> seg)
 {
 #if GEOS_DEBUG
-    cerr << "TaggedLineString[" << this << "] adding "
+    std::cerr << "TaggedLineString[" << this << "] adding "
          << " seg " << seg.get() << " to result"
-         << endl;
+         << std::endl;
 #endif
     resultSegs.push_back(seg.release());
 #if GEOS_DEBUG
-    cerr << "TaggedLineString[" << this << "] adding "
+    std::cerr << "TaggedLineString[" << this << "] adding "
          << " seg " << seg.get() << " to result"
-         << endl;
+         << std::endl;
 #endif
 }
 
+void
+TaggedLineString::removeRingEndpoint()
+{
+    auto* firstSeg = resultSegs.front();
+    auto* lastSeg = resultSegs.back();
+
+    firstSeg->p0 = lastSeg->p0;
+    delete lastSeg;
+    resultSegs.pop_back();
+}
+
 } // namespace geos::simplify
 } // namespace geos
