@@ -17,6 +17,7 @@
 #include <sstream>
 #include <string>
 #include <memory>
+#include <cfenv>
 
 namespace geos {
 namespace geom {
@@ -145,7 +146,7 @@ void object::test<3>
 ()
 {
     runTest("LINESTRING (0 0, 2 0)",
-            "MULTIPOINT (0 1, 1 0, 2 1)", 1.0);
+            "MULTIPOINT ((0 1), (1 0), (2 1))", 1.0);
 }
 
 // 4 - testLinesShowingDiscretenessEffect
@@ -182,6 +183,23 @@ void object::test<5>
 }
 
 
+// https://github.com/libgeos/geos/issues/515
+//
+// Avoid FE_INVALID floating point errors being raised
+//
+template<>
+template<>
+void object::test<6>
+()
+{
+    std::feclearexcept(FE_ALL_EXCEPT);
+
+    runTest(
+        "LINESTRING (0 0, 100 0, 10 100, 10 100)",
+        "LINESTRING (0 100, 0 10, 80 10)", 0.001, 47.89);
+
+    ensure("FE_INVALID raised", !std::fetestexcept(FE_INVALID));
+}
 
 // Crash on collection with empty components
 // https://github.com/libgeos/geos/issues/840
