@@ -20,8 +20,6 @@ namespace tut {
 // Common data used in test cases.
 struct test_capiremoverepeatedpoints_data : public capitest::utility {
     test_capiremoverepeatedpoints_data() {
-        GEOSWKTWriter_setTrim(wktw_, 1);
-        GEOSWKTWriter_setOutputDimension(wktw_, 3);
     }
 };
 
@@ -45,6 +43,17 @@ void object::test<1>
     ensure_geometry_equals(geom2_, expected_);
 }
 
+// https://github.com/libgeos/geos/issues/759
+template<>
+template<>
+void object::test<2>
+()
+{
+    geom1_ = GEOSGeomFromWKT("POLYGON ((0 0, 1 0, 1 1, 0 1, 0 0))");
+    result_ = GEOSRemoveRepeatedPoints(geom1_, 2.0);
+
+    ensure(result_ == nullptr);
+}
 
 
 } // namespace tut
