@@ -35,7 +35,6 @@
 #include <geos/geom/prep/PreparedGeometry.h>
 #include <geos/geom/prep/PreparedGeometryFactory.h>
 #include <geos/geom/util/Densifier.h>
-#include <geos/operation/overlay/OverlayOp.h>
 #include <geos/operation/overlay/snap/GeometrySnapper.h>
 #include <geos/operation/overlayng/OverlayNG.h>
 #include <geos/operation/overlayng/OverlayNGRobust.h>
@@ -93,6 +92,7 @@ using std::runtime_error;
 using geos::operation::overlayng::OverlayNG;
 using geos::operation::overlayng::UnaryUnionNG;
 using geos::operation::overlayng::OverlayNGRobust;
+using operation::valid::TopologyValidationError;
 
 namespace {
 
@@ -308,7 +308,7 @@ XMLTester::XMLTester()
     testValidOutput(false),
     testValidInput(false),
     sqlOutput(false),
-    HEXWKB_output(true)
+    HEXWKB_output(false)
 {
     setVerbosityLevel(0);
 }
@@ -571,7 +571,7 @@ XMLTester::testValid(const geom::Geometry* g, const std::string& label)
     operation::valid::IsValidOp ivo(g);
     bool valid = ivo.isValid();
     if(! valid) {
-        operation::valid::TopologyValidationError* err = ivo.getValidationError();
+        const TopologyValidationError* err = ivo.getValidationError();
         std::cerr << *curr_file << ":"
                   << " case" << caseCount << ":"
                   << " test" << testCount << ": "
@@ -703,6 +703,8 @@ XMLTester::parseCase(const tinyxml2::XMLNode* node)
         std::cout << *curr_file << ":";
         std::cout << " case" << caseCount << ":";
         std::cout << " skipped (" << thrownException << ")." << std::endl;
+        if (gA) delete gA;
+        if (gB) delete gB;
         return;
     }
 
@@ -1290,28 +1292,26 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
         else if(opName == "densify") {
             geom::Geometry* p_gT = gA;
 
-            GeomPtr gRes(parseGeometry(opRes, "expected"));
-            // gRes->normalize();
-
             geom::util::Densifier den(p_gT);
             double distanceTolerance = std::atof(opArg2.c_str());
             den.setDistanceTolerance(distanceTolerance);
             GeomPtr gRealRes = den.getResultGeometry();
+            gRealRes->normalize();
 
-            // gRealRes->normalize();
-
+            GeomPtr gRes(parseGeometry(opRes, "expected"));
+            gRes->normalize();
             if(gRes->compareTo(gRealRes.get()) == 0) {
                 success = 1;
             }
 
             actual_result = printGeom(gRealRes.get());
             expected_result = printGeom(gRes.get());
-
         }
 
 
         else if(opName == "union") {
             GeomPtr gRes(parseGeometry(opRes, "expected"));
+            gRes->normalize();
 
             profile.start();
 
@@ -1324,6 +1324,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
             }
 
             profile.stop();
+            gRealRes->normalize();
 
             success = checkOverlaySuccess(*gRes, *gRealRes);
 
@@ -1630,13 +1631,6 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
             gRes->normalize();
 
             GeomPtr gRealRes(p_gT->getCentroid());
-
-            if(gRealRes.get()) {
-                gRealRes->normalize();
-            }
-            else {
-                gRealRes = factory->createPoint();
-            }
             gRealRes->normalize();
 
             if(gRes->compareTo(gRealRes.get()) == 0) {
@@ -1717,7 +1711,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
 
 
             BufferOp op(p_gT, params);
-            gRealRes.reset(op.getResultGeometry(dist));
+            gRealRes = op.getResultGeometry(dist);
 
             profile.stop();
             gRealRes->normalize();
@@ -1761,8 +1755,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
             }
 
             BufferBuilder bufBuilder(params) ;
-            gRealRes.reset(bufBuilder.bufferLineSingleSided(
-                               p_gT, dist, leftSide)) ;
+            gRealRes = bufBuilder.bufferLineSingleSided(p_gT, dist, leftSide);
 
             profile.stop();
             gRealRes->normalize();
@@ -1803,7 +1796,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
             }
 
             BufferOp op(p_gT, params);
-            gRealRes.reset(op.getResultGeometry(dist));
+            gRealRes = op.getResultGeometry(dist);
 
             profile.stop();
             gRealRes->normalize();
@@ -1874,12 +1867,7 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
 
 
             auto polys = plgnzr.getPolygons();
-            std::vector<geom::Geometry*>* newgeoms = new std::vector<geom::Geometry*>;
-            for(unsigned int i = 0; i < polys.size(); i++) {
-                newgeoms->push_back(polys[i].release());
-            }
-
-            GeomPtr gRealRes(factory->createGeometryCollection(newgeoms));
+            GeomPtr gRealRes(factory->createGeometryCollection(std::move(polys)));
             gRealRes->normalize();
 
 
@@ -2216,10 +2204,16 @@ XMLTester::parseTest(const tinyxml2::XMLNode* node)
 
     }
     catch(const std::exception& e) {
-        std::cerr << "EXCEPTION on case " << caseCount
-                  << " test " << testCount << ": " << e.what()
-                  << std::endl;
-        actual_result = e.what();
+        if (expected_result == "exception") {
+            success = true;
+            actual_result = "exception";
+        }
+        else {
+            std::cerr << "EXCEPTION on case " << caseCount
+                      << " test " << testCount << ": " << e.what()
+                      << std::endl;
+            actual_result = e.what();
+        }
     }
     catch(...) {
         std::cerr << "Unknown EXEPTION on case "
@@ -2414,4 +2408,3 @@ main(int argC, char* argV[])
  * Revision 1.29  2006/03/17 14:56:39  strk
  * Fixed filename normalizer for sql output
  **********************************************************************/
-
